#!/bin/bash

# This script accepts a username and signal name on the
# command line and kills the jobs owned by that user using
# the given signal.
prog=${0##*/}
USAGE="Usage:  $prog -u username [-s name] [-n num] [-v]"

# Exit codes are:
#   1	Invalid option
#   2	Invalid username
#   3	Invalid signal name

username=""
signalopt="-s TERM"
signalname="TERM"
no_exec=""

while getopts u:s:n:v opt
do
    case "$opt" in
    u)	username="$OPTARG" ;;
    s)	signalopt="-s $OPTARG"; signalname="$OPTARG" ;;
    n)	signalopt="-n $OPTARG"; signalname="" ;;
    v)	no_exec="echo" ;;
    *)	echo >&2 "Invalid option: '-$opt'"
	echo >&2 "$USAGE"
	exit 1 ;;
    esac
done
shift $(( OPTIND-1 ))

# We should probably be verifying these data items at the
# time we process the option, but that would clutter up the
# 'case' statement. When we learn about shell functions we
# could use those to do the verification and the case
# statement would stay "clean looking"... (Left as an
# exercise for the student.)

# Verify that the username is valid.
# First check to make sure the user-supplied value will not
# cause grep(1) any headaches...
if [[ "$username" == *[^a-zA-Z0-9_]* ]]; then
    echo >&2 "Invalid username: '$username'"
    echo >&2 "$USAGE"
    exit 2
fi

# Now look through the password file...
if grep "^$username:" /etc/passwd >/dev/null
then
    true # the 'then' block cannot be completely empty
else
    echo >&2 "Invalid username: '$username'"
    echo >&2 "$USAGE"
    exit 2
fi

if [[ "$signalname" != "" ]]; then
    # Verify that the signal name is valid.
    typeset -l signameLowercase="$signalname"
    VALID="no"

    # This is ugly because of the hard-coded list, but
    # generating a list of valid signal names in a portable
    # manner is a pain.
    for name in hup int quit ill trap abrt emt fpe kill bus \
		segv sys pipe alrm term urg stop tstp cont \
		chld ttin ttou io xcpu xfsz vtalrm prof \
		winch info usr1 usr2
    do
	if [[ "$name" == "$signameLowercase" ]]; then
	    VALID="yes"
	    break
	fi
    done
    if [[ "$VALID" == "no" ]]; then
	echo >&2 "Invalid signal name: '$signal'"
	echo >&2 "$USAGE"
	exit 3
    fi
fi

# Why are there no quotes around $no_exec?
ps -u "$username" -o pid= | xargs $no_exec kill $signalopt
