#!/bin/bash

# This script loops around ten times and prints a message as well
# as appending a string to a temporary file.  The only option to
# the script specifies the loop iteration when ^C should be disabled.

prog=${0##*/}
USAGE="Usage:  $prog [-d loopIter]"

loop_start=1
loop_end=10
opt_d=3		# When to disable the SIGINT signal
while getopts d: opt
do
    case "$opt" in
    d)	opt_d="$OPTARG" ;;
    *)	echo >&2 "Invalid option: '$opt'"
	echo >&2 "$USAGE"
	exit 1 ;;
    esac
done
if (( opt_d < loop_start || opt_d > loop_end )); then
    echo >&2 "Value outside range of $loop_start-$loop_end"
    echo >&2 "$USAGE"
    exit 1
fi
clear

FILE=/tmp/frank
trap 'rm "$FILE"; echo "Removed $FILE ..."; exit' 2
date > "$FILE"

count=$loop_start
while (( count <= loop_end ))
do
    echo "Process ID $$, loop count $count"
    echo "Loop count is $count." >> "$FILE"
    let count=count+1
    if (( count > opt_d )); then
        trap '' 2
	echo "The script is ignoring SIGINT now."
    fi
    sleep 5
done
rm "$FILE"
